import unittest
from fate_crypto.hash import sm3_hash


class TestCorrect(unittest.TestCase):
    def test_hash_1(self):
        data = b"abc"
        expected = "66c7f0f462eeedd9d1f2d46bdc10e4e24167c4875cf2f7a2297da02b8f4ba8e0"
        self.assertEqual(sm3_hash(data).hex(), expected)

    def test_hash_2(self):
        data = b"abcdabcdabcdabcdabcdabcdabcdabcdabcdabcdabcdabcdabcdabcdabcdabcd"
        expected = "debe9ff92275b8a138604889c18e5a4d6fdb70e5387e5765293dcba39c0c5732"
        self.assertEqual(sm3_hash(data).hex(), expected)


if __name__ == "__main__":
    unittest.main()



from typing import overload

class Curve25519(object):
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, key): ...
    def get_private_key(self) -> bytes: ...
    def encrypt(self, m: bytes) -> bytes:
        """encrypt message.
        This contains flowing steps:
        1. Perform hashing to the group using the Elligator2 map to
            generate a Montgomery Point `A` in curve25519.
            (See https://tools.ietf.org/html/draft-irtf-cfrg-hash-to-curve-10#section-6.7.1)
        2. Perform scalar multipication `A * Scalar(secret)` to generate Montgomery Point `B`
        3. return `B` represents by 32 bytes value
        Args:
            m (bytes): message to encrypt

        Returns:
            bytes: encryptd message in 32-length bytes
        """
        ...
    def diffie_hellman(self, pub: bytes) -> bytes:
        """generate diffie_hellman like sharedsecret.

        Args:
            pub (bytes): encryted message in 32-length bytes.

        Returns:
            bytes: sharedsecret in 32-length bytes
        """
        ...



from fate_crypto.psi import Curve25519
import pickle
import unittest
import random


class TestStringMethods(unittest.TestCase):
    def test_ecdh(self):
        k1 = Curve25519()
        k2 = Curve25519()
        m = random.SystemRandom().getrandbits(33 * 8).to_bytes(33, "little")
        self.assertEqual(
            k2.diffie_hellman(k1.encrypt(m)), k1.diffie_hellman(k2.encrypt(m))
        )

    def test_pickle(self):
        k1 = Curve25519()
        m = random.SystemRandom().getrandbits(33 * 8).to_bytes(33, "little")
        pickled = pickle.dumps(k1)
        k2 = pickle.loads(pickled)
        self.assertEqual(k1.encrypt(m), k2.encrypt(m))


if __name__ == "__main__":
    unittest.main()



import random
import hashlib
from fate_crypto.psi import Curve25519


def ecdh(k, m):
    return k.encrypt(m)


def dh(k, e):
    return k.diffie_hellman(e)


def sha256(value):
    return hashlib.sha256(bytes(value, encoding="utf-8")).digest()


def test_ecdh_encrypt_bench(benchmark):
    k = Curve25519()
    m = random.SystemRandom().getrandbits(256).to_bytes(32, "little")
    result = benchmark(ecdh, k, m)


def test_ecdh_dh_bench(benchmark):
    k = Curve25519()
    m = random.SystemRandom().getrandbits(256).to_bytes(32, "little")
    e = k.encrypt(m)
    result = benchmark(dh, k, e)


def test_sha256_bench(benchmark):
    m = "1000000000"
    result = benchmark(sha256, m)
